import java.math.*;
import java.io.*;


public class SudokuSolver {
	public static void main(String args[]) {
		System.out.println("Sudoku-Solver von Simon v1.5\n");
		//SudokuDatabase.selfTest();
		if (args.length<=1) {
			System.err.println("Anwendung:");
			System.err.println("(1) Sudoku-Raetsel erzeugen:");
			System.err.println("    SudokuSolver -g Groesse [bx by] [Schwierigkeitsgrad 1-5]");
			System.err.println("");
			System.err.println("(2) Ausgefuelltes Sudoku-Raetsel erzeugen:");
			System.err.println("    SudokuSolver -f Groesse [bx by]");
			System.err.println("");
			System.err.println("(3) Sudoku-Raetsel pruefen:");
			System.err.println("    SudokuSolver -c Groesse [bx by] [Dateiname]");
			System.err.println("");
			System.err.println("(4) Sudoku-Raetsel loesen:");
			System.err.println("    SudokuSolver -s Groesse [bx by] [Dateiname]");
			System.err.println("    Anmerkung: Wird kein Dateiname angegeben, muss das Raetsel");
			System.err.println("               zeilenweise mit der Tastatur eingegeben werden.");
			System.err.println("               Ein Punkt . oder Stern * markiert eine freie Zelle.");
			System.err.println("");
			System.err.println("(5) Sudoku-Raetsel erzeugen und loesen:");
			System.err.println("    SudokuSolver -gs Groesse [bx by] [Schwierigkeitsgrad 1-5]");
			System.err.println("");
			
			System.err.println("Anmerkungen: o Die maximale untersttzte Groesse ist 64.");
			System.err.println("             o Die Angabe der Blockgroessen bx und by ist nur dann erlaubt,");
			System.err.println("               wenn das Sudoku kein Quadrat ist.");
			System.err.println("             o Die Raetseleingabe erfolgt bis Groesse 9 ohne Leerstellen");
			System.err.println("               zwischen den Ziffern. Bei groesseren Raetseln muessen die");
			System.err.println("               Zahlen mit Leerzeichen getrennt werden.");
			System.err.println("");
			return;
		}
		int gsize=9, bsx=3, bsy=3;
		if (args.length>=2) {
			gsize=Integer.valueOf(args[1]);
			System.out.println("Raetselgroesse: "+gsize);
			int t=(int)(Math.sqrt(gsize));
			if (args.length>=4) {
				bsx=Integer.valueOf(args[2]);
				bsy=Integer.valueOf(args[3]);
				System.out.println("Blockgroesse: "+bsx+"x"+bsy);
			}
			else if (t*t != gsize) {
				System.err.println("Blockbreite und Blockhoehe muessen angegeben werden.\nBeispiel: 'SudokuSolver -g 15 5 3'");
				return;
			}
			else {
				bsx=t; bsy=t;
			}
            Sudoku.init(gsize, bsx, bsy);
        }
		else {
			System.err.println("Weitere Angaben erforderlich!");
			return;
		}
		if (args[0].equals("-g") || args[0].equals("-gs")) {
			int diff=3;
			if (args.length==3 || args.length==5)
				diff = Integer.parseInt(args[args.length-1]);
			System.out.println("Schwierigkeitsgrad: "+diff);
			if (!Sudoku.createGameDb(diff))
				Sudoku.createGame(diff, false);
		}
		else if (args[0].equals("-f")) {
			Sudoku.createGame(-1, true);
		}
		else if (args[0].equals("-s")) {
			if (args.length==3 || args.length==5) {
				System.out.println("Einlesen aus Datei: "+args[args.length-1]);
				if (gsize<=9) {
					if (!Sudoku.readFromFile(args[args.length-1])) return;
				}
				else {
					if (!Sudoku.readFromFileWide(args[args.length-1])) return;
				}
			}
			else
				if (!Sudoku.readFromInput()) return;
		}
		else if (args[0].equals("-c")) {
			if (args.length==3 || args.length==5) {
				System.out.println("Einlesen aus Datei: "+args[args.length-1]);
				if (gsize<=16) {
					if (!Sudoku.readFromFile(args[args.length-1])) return;
				}
				else {
					if (!Sudoku.readFromFileWide(args[args.length-1])) return;
				}
			}
			else
				if (!Sudoku.readFromInput()) return;
		}
		else {
			System.err.println("Modus "+args[0]+" unbekannt!");
			return;
		}
		Sudoku start = new Sudoku(Sudoku.sp_init, "Init", 0);
		System.out.println("\nRaetsel:");
		start.printSolution();
		if (args[0].equals("-s") || args[0].equals("-gs")) {
			start.work();
			System.out.print("Fertig. ");
			if (Sudoku.solution>1)
				System.out.println(Sudoku.solution+" Loesungen gefunden!");
			else if (Sudoku.solution==1)
				System.out.println("1 Loesung gefunden!");
			else
				System.out.println("Keine Loesung gefunden!");
			if (Sudoku.maxLevel>0)
				System.out.println("Backtracking war erforderlich. Maximale Suchtiefe: "+Sudoku.maxLevel);
		}
		else if (args[0].equals("-c")) {
			if (Sudoku.checkGame(Sudoku.sp_init, gsize, bsx, bsy))
				System.out.println("Raetsel scheint gueltig zu sein.");
			else
				System.out.println("Raetsel ungueltig.");
		}
	}
}


class Sudoku {
	public static final boolean debug = false;

	public static int GAMESIZE = 64;
	public static int BLOCKSIZE_X = 8;
	public static int BLOCKSIZE_Y = 8;
	public static int NUMBEROFFSET = 1;
	public static int solution=0;
	public static int maxLevel=0;
	public static final boolean USEBLOCKS = true;
	public static final boolean ADVANCED = true;

	public static int[][] sp_init = new int[GAMESIZE][GAMESIZE];
    private static int[] blkx = new int[GAMESIZE], blky = new int[GAMESIZE];
    private static int cx, cy;

	private int[][] sp = new int[GAMESIZE][GAMESIZE];	// Spielfeld
	private int[][] spcnt = new int[GAMESIZE][GAMESIZE];	// Anzahl Mglichkeiten
	private long[] rowid = new long[GAMESIZE], colid = new long[GAMESIZE], blockid = new long[GAMESIZE];
	private String desc;
	private int level;
	private boolean ungueltig=false;


	public Sudoku(int[][] gsp, String s, int l) {
		desc=s;
		level=l;
		if (l>maxLevel) maxLevel=l;
		for (int x=0; x<GAMESIZE; x++) 
			for (int y=0; y<GAMESIZE; y++)
				sp[y][x] = gsp[y][x];
        if (debug && !checkGame(sp, GAMESIZE, BLOCKSIZE_X, BLOCKSIZE_Y))
            System.err.println("Neues Spiel ist ungueltig!");
	}


    public static void init(int gsize, int bsizex, int bsizey) {
        GAMESIZE=gsize;
        solution=0;
        sp_init=new int[GAMESIZE][GAMESIZE];
        blkx=new int[GAMESIZE];
        blky=new int[GAMESIZE];
        BLOCKSIZE_X = bsizex;//(int)(Math.sqrt(GAMESIZE));
        BLOCKSIZE_Y = bsizey;
		for (int i=0; i<GAMESIZE; i++) {
			blkx[i]=(i%(GAMESIZE/BLOCKSIZE_X))*BLOCKSIZE_X;
			blky[i]=(i/(GAMESIZE/BLOCKSIZE_X))*BLOCKSIZE_Y;
		}
    }
    

	private static void switchBlockRows(int r1, int r2) {
		if (r1==r2) return;
		for (int y=0; y<BLOCKSIZE_Y; y++)
			for (int x=0; x<GAMESIZE; x++)
			{
				int temp=sp_init[y+r1*BLOCKSIZE_Y][x];
				sp_init[y+r1*BLOCKSIZE_Y][x]=sp_init[y+r2*BLOCKSIZE_Y][x];
				sp_init[y+r2*BLOCKSIZE_Y][x]=temp;
			}
	}
	
	
	private static void switchBlockCols(int r1, int r2) {
		if (r1==r2) return;
		for (int x=0; x<BLOCKSIZE_X; x++)
			for (int y=0; y<GAMESIZE; y++)
			{
				int temp=sp_init[y][x+r1*BLOCKSIZE_X];
				sp_init[y][x+r1*BLOCKSIZE_X]=sp_init[y][x+r2*BLOCKSIZE_X];
				sp_init[y][x+r2*BLOCKSIZE_X]=temp;
			}
	}
	
	
	private static void switchRows(int r1, int r2, int blk) {
		if (r1==r2) return;
		for (int x=0; x<GAMESIZE; x++)
		{
			int temp=sp_init[r1+blk*BLOCKSIZE_Y][x];
			sp_init[r1+blk*BLOCKSIZE_Y][x]=sp_init[r2+blk*BLOCKSIZE_Y][x];
			sp_init[r2+blk*BLOCKSIZE_Y][x]=temp;
		}
	}
	
	
	private static void switchCols(int r1, int r2, int blk) {
		if (r1==r2) return;
		for (int y=0; y<GAMESIZE; y++)
		{
			int temp=sp_init[y][r1+blk*BLOCKSIZE_X];
			sp_init[y][r1+blk*BLOCKSIZE_X]=sp_init[y][r2+blk*BLOCKSIZE_X];
			sp_init[y][r2+blk*BLOCKSIZE_X]=temp;
		}
	}
	
	
	public static void createGame(int diff, boolean full) {
		System.out.println("Das Raetsel wird zufaellig erzeugt. Es kann nicht spielbar sein!");
		// Erstellt gltiges Rtsel
		int cnt=0;
		for (int y=0; y<Sudoku.GAMESIZE; y++) {
			if (y%Sudoku.BLOCKSIZE_Y==0) cnt=y/Sudoku.BLOCKSIZE_Y;
			for (int x=0; x<Sudoku.GAMESIZE; x++) {
				Sudoku.sp_init[y][x]=(cnt++)%(GAMESIZE);
			}
			cnt+=Sudoku.BLOCKSIZE_X;
		}
		// Erstellt das Nummermapping
		int[] nm = new int[GAMESIZE];
		for (int i=0; i<GAMESIZE; i++)
			nm[i]=i;
		for (int i=0; i<256; i++) {
			int r1=(int)(Math.random()*1.0*(double)GAMESIZE);
			int r2=(int)(Math.random()*1.0*(double)GAMESIZE);
			if (r1!=r2) {
				int t=nm[r1]; nm[r1]=nm[r2]; nm[r2]=t;
			}
		}
		// Wendet Nummermapping an
        for (int y=0; y<GAMESIZE; y++)
            for (int x=0; x<GAMESIZE; x++)
           		sp_init[y][x]=nm[sp_init[y][x]];
		// Lscht Zellen
		if (!full) {
/*			int i=0;
			if (diff==-1) diff=3;
			double d=4.0-diff/2.5;
			while ((double)i<(double)GAMESIZE*GAMESIZE/d) {
				int rx=(int)(Math.random()*1.0*(double)GAMESIZE);
				int ry=(int)(Math.random()*1.0*(double)GAMESIZE);
				if (sp_init[ry][rx]!=-1) {
					i++;
					sp_init[ry][rx]=-1;
				}
			}
        	sp_init[0][0]=-1;
        	sp_init[1][1]=-1;*/
			cnt=0;
			int d=2;
			if (diff<=2) d=3;
			for (int y=0; y<GAMESIZE; y++) {
				cnt = y%d;
				for (int x=0; x<GAMESIZE; x++) {
					if (cnt%d==0)
						sp_init[y][x]=-1;
					cnt++;
				}
			}
		}
        // Vertauscht Zeilen/Spalten/Blcke
		for (int c=0; c<GAMESIZE; c++) {
			for (int i=0; i<GAMESIZE; i++) 
				switchBlockRows((int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y));
			for (int i=0; i<GAMESIZE; i++) 
				switchBlockCols((int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X));
			for (int i=0; i<GAMESIZE; i++) 
				switchRows((int)(Math.random()*1.0*(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y));
			for (int i=0; i<GAMESIZE; i++) 
				switchCols((int)(Math.random()*1.0*(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X));
		}
    }
	
	
	public static boolean createGameDb(int diff) {
		// Ldt Spiel aus Datenbank
		boolean rot=false;
		if (BLOCKSIZE_X<BLOCKSIZE_Y) rot=true;
		if (!SudokuDatabase.loadGameFromDb(sp_init, GAMESIZE, rot, diff))
			return false;
		
		int[] nm = new int[GAMESIZE];
		// Erstellt das Nummermapping
		for (int i=0; i<GAMESIZE; i++)
			nm[i]=i;
		for (int i=0; i<256; i++) {
			int r1=(int)(Math.random()*1.0*(double)GAMESIZE);
			int r2=(int)(Math.random()*1.0*(double)GAMESIZE);
			if (r1!=r2) {
				int t=nm[r1]; nm[r1]=nm[r2]; nm[r2]=t;
			}
		}
		// Wendet Nummermapping an
        for (int y=0; y<GAMESIZE; y++) {
            for (int x=0; x<GAMESIZE; x++) {
            	if (sp_init[y][x]!=-1)
            		sp_init[y][x]=nm[sp_init[y][x]];
            }
        }
        // Zuflliges Austauschen der Zeilen/Spalten/Blcke
		for (int c=0; c<GAMESIZE; c++) {
			for (int i=0; i<GAMESIZE; i++) 
				switchBlockRows((int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y));
			for (int i=0; i<GAMESIZE; i++) 
				switchBlockCols((int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X));
			for (int i=0; i<GAMESIZE; i++) 
				switchRows((int)(Math.random()*1.0*(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)BLOCKSIZE_Y), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_Y));
			for (int i=0; i<GAMESIZE; i++) 
				switchCols((int)(Math.random()*1.0*(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)BLOCKSIZE_X), (int)(Math.random()*1.0*(double)GAMESIZE/(double)BLOCKSIZE_X));
		}
		return true;
    }
	
	
    public static boolean checkGame(int[][] game, int gsize, int bsizex, int bsizey) {
//        int gsize=game[0].length();
        //int bsize=(int)(Math.sqrt(gsize));
        for (int y=0; y<gsize; y++) {
            BigInteger c=BigInteger.ZERO;
            for (int x=0; x<gsize; x++) {
                int l=game[y][x];
                if (l >= 0) {
                    if (!c.testBit(l))
                        c.setBit(l);
                    else
                        return false;
                }
            }
        }
        for (int x=0; x<gsize; x++) {
            BigInteger c=BigInteger.ZERO;
            for (int y=0; y<gsize; y++) {
                int l=game[y][x];
                if (l >= 0) {
                    if (!c.testBit(l))
                        c.setBit(l);
                    else
                        return false;
                }
            }
        }
        if (USEBLOCKS) {
            for (int blk=0; blk<gsize; blk++) {
                BigInteger c=BigInteger.ZERO;
                for (int y=0; y<bsizey; y++) {
                    for (int x=0; x<bsizex; x++) {
                        int l=game[y+blky[blk]][x+blkx[blk]];
                        if (l >= 0) {
                            if (!c.testBit(l))
                                c.setBit(l);
                            else
                                return false;
                        }
                    }
                }
            }
        }
        return true;
    }
    
    
	public static boolean readFromFile(String fn) {
		File f = new File(fn);
		int cnt=0, o=0;
		char[] buffer = new char[(int)f.length()];
		try {
			InputStream in = new FileInputStream(f);
			while (cnt<f.length()) {
				char b = (char)in.read();
				if ((b>='1' && b<='9') || (b>='a' && b<='f') || (b>='A' && b<='F') || b == '.' || b == '*')
					buffer[o++] = b;
				cnt++;
			}
		} catch (Exception e) {System.err.println("Fehler beim Lesen von "+fn); return false;}
		if (o==0 || o>GAMESIZE*GAMESIZE) {
			System.err.println("Eingabe ungueltig!");
			return false;
		}
		for (int y=0; y<GAMESIZE; y++)
			for (int x=0; x<GAMESIZE; x++) {
				if (buffer[x+y*GAMESIZE] != '.' && buffer[x+y*GAMESIZE] != '*') {
					String s=""+buffer[x+y*GAMESIZE];
					sp_init[y][x]=Integer.parseInt(s, 16)-NUMBEROFFSET;
				}
				else
					sp_init[y][x]=-1;
			}
		return true;
	}
	
	
	private static int eatNumber(InputStream in) {
		int t;
		String s="";
		try {
			do
				t = in.read();
			while (t != -1 && (t<(int)('0') || t>(int)('9')) && t!=(int)('*') && t!=(int)('.'));
			if (t == -1) return -2;
			while (t != -1 && (t>=(int)('0') && t<=(int)('9')) || t==(int)('.') || t==(int)('*')) {
				if (t>0) s+=(char)t;
				t = in.read();
			}
			if (t == -1) return -2;
			if (s.equals("*") || s.equals(".")) return -1;
			else return Integer.parseInt(s)-NUMBEROFFSET;
		} catch (Exception e) {System.err.println("Fehler beim Lesen der Datei");}
		return -1;
	}
	
	public static boolean readFromFileWide(String fn) {
		File f = new File(fn);
		int o=0;
		int[] buffer = new int[(int)f.length()];
		try {
			InputStream in = new FileInputStream(f);
			while (true) {
				int t = eatNumber(in);
				if (t >= -1) buffer[o++] = t;
				else break;
			}
		} catch (Exception e) {System.err.println("Fehler beim Lesen von "+fn); return false;}
		if (o==0 || o>GAMESIZE*GAMESIZE) {
			System.err.println("Eingabe ungueltig!");
			return false;
		}
		for (int y=0; y<GAMESIZE; y++)
			for (int x=0; x<GAMESIZE; x++)
				sp_init[y][x]=buffer[x+y*GAMESIZE];
		return true;
	}
	
	
	public static boolean readFromInput() {
		int o=0;
		int[] buffer = new int[GAMESIZE*GAMESIZE];
		try {
			InputStream in = System.in;
			while (true) {
				int t = eatNumber(in);
				if (t >= -1) buffer[o++] = t;
				else break;
				if (o==GAMESIZE*GAMESIZE) break;
			}
		} catch (Exception e) {System.err.println("Fehler"); return false;}
		if (o==0 || o>GAMESIZE*GAMESIZE) {
			System.err.println("Eingabe ungueltig!");
			return false;
		}
		for (int y=0; y<GAMESIZE; y++)
			for (int x=0; x<GAMESIZE; x++)
				sp_init[y][x]=buffer[x+y*GAMESIZE];
		return true;
	}
	
	
    private boolean checkRow(int row) {
        int[] anz = new int[GAMESIZE], lpos = new int[GAMESIZE];
        boolean c=false;
        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lpos[i] = 0;}
        for (int x=0; x<GAMESIZE; x++) if (spcnt[row][x]>1) {
            long temp = schnittmenge(row, x);
            for (int i=0; i<GAMESIZE; i++)
                if ((temp&(1L<<(long)i))!=0) {anz[i]++; lpos[i]=x;}
        }
        for (int i=0; i<GAMESIZE; i++)
            if (anz[i] == 1) {sp[row][lpos[i]] = i; spcnt[row][lpos[i]]=1; cy=row; cx=lpos[i]; return true;}
        return c;
    }
    
    
    private boolean checkCol(int col) {
        int[] anz = new int[GAMESIZE], lpos = new int[GAMESIZE];
        boolean c=false;
        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lpos[i] = 0;}
        for (int y=0; y<GAMESIZE; y++) if (spcnt[y][col]>1) {
            long temp = schnittmenge(y, col);
            for (int i=0; i<GAMESIZE; i++)
                if ((temp&(1L<<(long)i))!=0) {anz[i]++; lpos[i]=y;}
        }
        for (int i=0; i<GAMESIZE; i++)
            if (anz[i] == 1) {sp[lpos[i]][col] = i; spcnt[lpos[i]][col]=1; cy=lpos[i]; cx=col; return true;}
        return c;
    }
    
    
    private boolean checkBlock(int blk) {
        int[] anz = new int[GAMESIZE], lposx = new int[GAMESIZE], lposy = new int[GAMESIZE];
        boolean c=false;
        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lposx[i] = 0; lposy[i] = 0;}
        for (int y=0; y<BLOCKSIZE_Y; y++) {
            for (int x=0; x<BLOCKSIZE_X; x++) if (spcnt[y+blky[blk]][x+blkx[blk]]>1) {
                long temp = schnittmenge(y+blky[blk], x+blkx[blk]);
                for (int i=0; i<GAMESIZE; i++)
                    if ((temp&(1L<<(long)i))!=0) {anz[i]++; lposx[i]=x+blkx[blk]; lposy[i]=y+blky[blk];}
            }
        }
        for (int i=0; i<GAMESIZE; i++)
            if (anz[i] == 1) {sp[lposy[i]][lposx[i]] = i; spcnt[lposy[i]][lposx[i]]=1; cy=lposy[i]; cx=lposx[i]; return true;}
        return c;
    }
    
    
	private void remrowid(int row, int id) {
		if (id<0 || id>GAMESIZE-1 || row<0 || row>GAMESIZE-1) 
			return;
//		if (!ungueltig && ((rowid[row]>>>id)&1) == 0)
//			ungueltig=true;
		rowid[row] &= ~(long)(1L<<(long)id);
	}


	private void remcolid(int col, int id) {
		if (id<0 || id>GAMESIZE-1 || col<0 || col>GAMESIZE-1) 
			return;
//		if (!ungueltig && ((colid[col]>>>id)&1) == 0)
//			ungueltig=true;
		colid[col] &= ~(long)(1L<<(long)id);
	}


	private void remblockid(int blk, int id) {
		if (id<0 || id>GAMESIZE-1 || blk<0 || blk>GAMESIZE-1) 
			return;
//		if (!ungueltig && ((blockid[blk]>>>id)&1) == 0)
//			ungueltig=true;
		blockid[blk] &= ~(long)(1L<<(long)id);
	}


	private static int getBlock(int row, int col) {
		return col/BLOCKSIZE_X+(row/BLOCKSIZE_Y)*BLOCKSIZE_Y;
	}


	private long schnittmenge(int row, int col) {
		int blk=getBlock(row, col);
		if (row<0 || row>GAMESIZE-1 || col<0 || col>GAMESIZE-1) 
			return -1L;
		if (USEBLOCKS)
			return rowid[row] & colid[col] & blockid[blk];
		else
			return rowid[row] & colid[col];
	}


	private static int countbits(long id) {
		int erg=0;
		for (int i=0; i<GAMESIZE; i++)
			if (((long)id&(long)(1L<<(long)i)) != 0L) 
				erg++;
//        if (id<0) erg++;
		return erg;
	}


	private static int bitnummer(long id) {
		if (id==0) return -1;
		int erg=0;
		while (((long)id&1L) == 0L) {
			id=id>>>1L;
			erg++;
		}
		return erg;
	}
	
	
	private static String sprintPool(long d) {
		String s="";
        int erg=0;
        while (d != 0L && erg < GAMESIZE) {
            if ((d&1L) == 1L)
                s=s + (erg+NUMBEROFFSET) + ", ";
            d>>>=1L;
            erg++;
        }
        return s.substring(0,s.length()-2);
	}
    
    
	private static void printSeparator() {
		String fmt2="---";
		if (GAMESIZE >= 1000) {fmt2="------";}
		else if (GAMESIZE >= 100) {fmt2="-----";}
		else if (GAMESIZE >= 10) {fmt2="----";}
		for (int c=0; c<GAMESIZE/BLOCKSIZE_X; c++) {
			System.out.print("+");
			for (int i=0; i<BLOCKSIZE_X; i++)
				System.out.print(fmt2);//"----");
		}
		System.out.println("+");
	}
	
	
	public void printSolution() {
		int fc=0;
		String fmt="%1d", fmt2="*";
		if (GAMESIZE >= 1000) {fmt="%4d"; fmt2="   *";}
		else if (GAMESIZE >= 100) {fmt="%3d"; fmt2="  *";}
		else if (GAMESIZE >= 10) {fmt="%2d"; fmt2=" *";}
		for (int y=0; y<GAMESIZE; y++) {
			if (USEBLOCKS && y%BLOCKSIZE_Y==0) printSeparator();
			for (int x=0; x<GAMESIZE; x++) {
				if (USEBLOCKS && x%BLOCKSIZE_X==0) System.out.print("|");
				if (sp[y][x]>=0) {
					//String s=BigInteger.valueOf(sp[y][x]+NUMBEROFFSET).toString(10);
					String s=String.format(fmt, sp[y][x]+NUMBEROFFSET);
                    //if (s.length() == 1) System.out.print("  ");
                    //else if (s.length() == 2) System.out.print(" ");
					System.out.print(" "+s+" ");
				}
				else {
//					String s=String.format(fmt2);
					System.out.print(" "+fmt2+" ");//"  * ");
					fc++;
				}
			}
			if (USEBLOCKS) System.out.print("|");
			System.out.println("");
		}
		if (USEBLOCKS) printSeparator();
		if (fc>0)
			System.out.println("Freie Zellen: "+fc+"\n");
		else
			System.out.println("");
	}


	private boolean berechne() {
		boolean change, done;
		do {
			done=true;
			change=false;
			for (int i=0; i<GAMESIZE; i++) {
				rowid[i]=0xffffffffffffffffL;
				colid[i]=0xffffffffffffffffL;
				blockid[i]=0xffffffffffffffffL;
			}
			for (int y=0; y<GAMESIZE; y++)
				for (int x=0; x<GAMESIZE; x++)
					if (sp[y][x]>=0) 
						spcnt[y][x]=1;
					else 
						spcnt[y][x]=GAMESIZE;
			for (int y=0; y<GAMESIZE; y++)
				for (int x=0; x<GAMESIZE; x++) 
					remrowid(y, sp[y][x]);
			for (int x=0; x<GAMESIZE; x++) 
				for (int y=0; y<GAMESIZE; y++)
					remcolid(x, sp[y][x]);
			if (USEBLOCKS)
				for (int blk=0; blk<GAMESIZE; blk++) 
					for (int y=0; y<BLOCKSIZE_Y; y++)
						for (int x=0; x<BLOCKSIZE_X; x++) {
                            //System.out.println("blk="+blk+", x="+x+", y="+y+", blky[blk]="+blky[blk]+", blkx[blk]="+blkx[blk]);
							remblockid(blk, sp[y+blky[blk]][x+blkx[blk]]);
                        }
			if (ADVANCED) {
				for (int y=0; y<GAMESIZE; y++) {
					change = checkRow(y);
					if (change)
						break;
				}
				if (!change) for (int x=0; x<GAMESIZE; x++) {
					change = checkCol(x);
					if (change)
						break;
				}
				if (!change) for (int b=0; b<GAMESIZE; b++) {
					change = checkBlock(b);
					if (change)
						break;
				}
			}
			if (!change) 
              for (int y=0; y<GAMESIZE; y++) {
				for (int x=0; x<GAMESIZE; x++) {
					if (spcnt[y][x]>1) {
						int t=countbits(schnittmenge(y, x));
						if (t==0) {
							if (debug) {
								String s="";
								for (int c=0; c<level; c++)
									s=s+" ";
								System.out.println(s+"-> w1 Failed ("+desc.trim()+")");
							}
							return false;
						}
						else if (t==1) {
							sp[y][x]=bitnummer(schnittmenge(y,x));
                            spcnt[y][x]=1;
							change=true; break;
						}
						else 
							done=false;
						spcnt[y][x]=t;
					}
				}
				if (change) 
					break;
			}
		}
		while (change);
		if (done) {
			solution++;
			System.out.println("Loesung "+solution+":");
			if (debug && checkGame(sp, GAMESIZE, BLOCKSIZE_X, BLOCKSIZE_Y)) System.out.println("<<Loesung gueltig>>");
			printSolution();
		}
		return true;
	}


	private boolean berechne(int gx, int gy) {
		boolean change, done;
		do {
			done=true;
			change=false;
            remrowid(gy, sp[gy][gx]);
            remcolid(gx, sp[gy][gx]);
			if (USEBLOCKS)
                remblockid(getBlock(gy, gx), sp[gy][gx]);
            if (ADVANCED) {
            	change=checkRow(gy); if (change) {gy=cy; gx=cx;}
            	if (!change) {change=checkCol(gx); if (change) {gy=cy; gx=cx;}}
            	if (!change) {change=checkBlock(getBlock(gy, gx)); if (change) {gy=cy; gx=cx;}}
            }
			if (!change) for (int y=0; y<GAMESIZE; y++) {
				for (int x=0; x<GAMESIZE; x++) {
					if (spcnt[y][x]>1) {
						int t=countbits(schnittmenge(y,x));
						if (t==0) {
							if (debug) {
								String s="";
								for (int c=0; c<level; c++)
									s=s+" ";
								System.out.println(s+"-> w2 Failed ("+desc.trim()+")");
							}
							return false;
						}
						else if (t==1) {
							sp[y][x]=bitnummer(schnittmenge(y,x)); 
                            spcnt[y][x]=1;
                            gy=y; gx=x;
							change=true; break;
						}
						else 
							done=false;
						spcnt[y][x]=t;
					}
				}
				if (change) 
					break;
			}
		}
		while (change);
		if (done) {
			solution++;
			System.out.println("Loesung "+solution+":");
			if (debug && checkGame(sp, GAMESIZE, BLOCKSIZE_X, BLOCKSIZE_Y)) System.out.println("<<Loesung gueltig>>");
			printSolution();
		}
		return true;
	}


	public void work() {
		if (!berechne()) {
			if (ungueltig)
				System.out.println("Eingabedaten ungueltig!");
			return;
		}
		int mx=0, my=0, minsp=GAMESIZE;
		for (int y=0; y<GAMESIZE; y++) {
			for (int x=0; x<GAMESIZE; x++) {
				if (spcnt[y][x]>1 && spcnt[y][x]<minsp) {
					mx=x; my=y; minsp=spcnt[y][x];
                    if (minsp==2) break;
				}
			}
            if (minsp==2) break;
		}
		long temp=schnittmenge(my,mx);
		for (int i=0; i<GAMESIZE; i++) 
			if (((temp>>>i)&1)==1)
			{
				sp[my][mx]=i;
				String s="";
				if (debug) {
					for (int c=0; c<level; c++)
						s=s+" ";
					s=s+"Start new w1@"+mx+","+my+" with "+(i+NUMBEROFFSET)+" of ["+sprintPool(temp)+"]";
					System.out.println(s);
				} 
				else s="";
				Sudoku next = new Sudoku(sp, s, level+1);
				next.work(spcnt, rowid, colid, blockid, mx, my);
				//if (ungueltig) return;
			}
	}
    
    
	public void work(int[][] gspcnt, long[] grow, long[] gcol, long[] gblk, int gx, int gy) {
        for (int y=0; y<GAMESIZE; y++) {
            for (int x=0; x<GAMESIZE; x++) {
                spcnt[y][x] = gspcnt[y][x];
            }
            colid[y]=gcol[y];
            rowid[y]=grow[y];
            blockid[y]=gblk[y];
        }
        spcnt[gy][gx]=1;
        
		if (!berechne(gx, gy)) {
			if (ungueltig)
				System.out.println("Eingabedaten ungueltig!");
			return;
		}
		int mx=0, my=0, minsp=GAMESIZE;
		for (int y=0; y<GAMESIZE; y++) {
			for (int x=0; x<GAMESIZE; x++) {
				if (spcnt[y][x]>1 && spcnt[y][x]<minsp) {
					mx=x; my=y; minsp=spcnt[y][x];
                    if (minsp==2) break;
				}
			}
            if (minsp==2) break;
		}
		long temp=schnittmenge(my,mx);
		for (int i=0; i<GAMESIZE; i++) 
			if (((temp>>>i)&1)==1)
			{
				sp[my][mx]=i;
				String s="";
				if (debug) {
					for (int c=0; c<level; c++)
						s=s+" ";
					s=s+"Start new w2@"+mx+","+my+" with "+(i+NUMBEROFFSET)+" of ["+sprintPool(temp)+"]";
					System.out.println(s);
				} 
				else s="";
				Sudoku next = new Sudoku(sp, s, level+1);
				next.work(spcnt, rowid, colid, blockid, mx, my);
				//if (ungueltig) return;
			}
	}
}

